#include "DataExportPipe.h"
#include <stdexcept>

using namespace std;

const string kProtocolMarker = "squid_data_export_pipe";
const int kProtocolVersion = 1;
const VirtualSocketPath kSocketName("squid_export");


/////////////////////////////////////////////////////////////

DataExportSocket::DataExportSocket()
{
    mSocket = std::unique_ptr<UnixSocket>(new UnixSocket(kSocketName, UnixSocket::Client));

    mSocket->writeString(kProtocolMarker);
    mSocket->writeValue(int(kProtocolVersion));
}

void DataExportSocket::sendRequest(const std::string& ipAddr, const std::string& httpHeader)
{
    mSocket->writeString(ipAddr);
    mSocket->writeString(httpHeader);

    string rep;
    mSocket->readString(rep);
    if (rep != "ok")
        throw runtime_error("Invalid acknowledgement response");
}


/////////////////////////////////////////////////////////////

DataImportServerConnection::DataImportServerConnection(std::unique_ptr<UnixSocket> socket)
{
    mSocket = std::move(socket);    
}

void DataImportServerConnection::initConnection()
{
    //  get protocol marker
    string protocolMarker;
    mSocket->readString(protocolMarker);
    if (protocolMarker != kProtocolMarker)
        throw runtime_error("Protocol maker not found");

    // get version number
    int version;
    mSocket->readValue(version);
    if (version != kProtocolVersion)
        throw runtime_error("Got invalid or unsupported protocol version");
}

void DataImportServerConnection::receiveRequest(std::string& outIpAddr, std::string& outHttpHeader)
{
    mSocket->readString(outIpAddr);
    mSocket->readString(outHttpHeader);   
}


/////////////////////////////////////////////////////////////

DataImportServerSocket::DataImportServerSocket()
{
    mSocket = std::unique_ptr<UnixSocket>(new UnixSocket(kSocketName, UnixSocket::Server));    
}

std::unique_ptr<DataImportServerConnection> DataImportServerSocket::acceptConnection()
{
    std::unique_ptr<UnixSocket> newSocket = std::move(mSocket->acceptNewConnection());
    return std::unique_ptr<DataImportServerConnection>(new DataImportServerConnection(std::move(newSocket)));
}