#include "NamedPipe.h"
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdexcept>
#include <string.h>


void DataExportCommsBase::writeString(const std::string& str)
{    
    writeValue(str.length());
    write(str.c_str(), str.length());
}

void DataExportCommsBase::readString(std::string& outStr)
{
    size_t length;
    readValue(length);

    // sanity check length, impose a 64k limit!
    if (length > 65536)
        throw std::runtime_error("string length exceeded limit, value is invalid or data length is too long");

    outStr.resize(length);
    read(&outStr[0], length);
}

//////////////////////////////////////////////


NamedPipe::NamedPipe(const std::string& pipeName, bool isOwner)
{
    mPipeName = pipeName;
    mIsOwner = isOwner;

    if (mIsOwner)
    {
        // create the FIFO (named pipe)
        int res = mkfifo(pipeName.c_str(), 0777/*0666*/);
        if (res)
            throw std::runtime_error(std::string("Failed to created named pipe ") + strerror(errno));
    }

    mFileDescriptor = open(pipeName.c_str(), O_RDWR);
    if (mFileDescriptor == -1)
        throw std::runtime_error("Failed to create pipe");
}

void NamedPipe::write(const void* data, size_t length)
{
    // TODO: check return
    ::write(mFileDescriptor, data, length);
}

void NamedPipe::read(void* outData, size_t length)
{
    // TODO: check return
    ::read(mFileDescriptor, outData, length);
}

NamedPipe::~NamedPipe()
{
    close(mFileDescriptor);

    if (mIsOwner)
        unlink(mPipeName.c_str());
}